/***************************************************************************
**    tndLock.c
**    Termnetd Device Locking Functions
**
** Copyright (C) 1996      Joseph Croft <jcroft@unicomp.net>  
** 
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 1, or (at your option)
** any later version.
** 
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
** 
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software
** Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
**
 ***************************************************************************/
#include "../sysdefs.h"
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <memory.h>
#include <setjmp.h>
#include <syslog.h>
#include "termnetd.h"

static int dbg = 0;

int lockDevc(portConf *pDevc)
{
   devcLock *pEntry;
   int err, rv = -1;
   jmp_buf mEnv;

   if (dbg) syslog(LOG_DEBUG, "lockDevc():Enter");
   if (!isLocked(pDevc->devc))
   {
      if (dbg) syslog(LOG_DEBUG, "lockDevc():Device %s not locked",
                      pDevc->devc);
      if ((err = setjmp(mEnv)) == 0)
      {
         if ((pEntry = (devcLock *)malloc(sizeof(devcLock))) != NULL)
         {
            if (dbg) syslog(LOG_DEBUG, "lockDevc():Locking Device %s",
                                       pDevc->devc);
            pEntry->pNext = pLockList;
            pEntry->pPrev = NULL;
            if (pLockList)
               pLockList->pPrev = pEntry;
            pLockList = pEntry;
            pEntry->pDevc = pDevc; 
         }
      }
      else if (err != 0)
      {
         syslog(LOG_ERR, "Quiting: Out of memory!!!!\n");
         exit(1);
      }
      rv = 0;
   }
   else
      rv = -1;
   if (dbg) syslog(LOG_DEBUG, "lockDevc():Exit");
   return(rv);
}

void ulckDevc(pid_t pid)
{
   devcLock *pEntry;

   if (dbg) syslog(LOG_DEBUG, "ulckDevc():Enter");
   for (pEntry = pLockList; pEntry != NULL; pEntry = pEntry->pNext)
   {
      if (dbg) syslog(LOG_DEBUG, "ulckDevc():Testing Entries %d & %d",
                      pEntry->pDevc->pid, pid);
      if (pid == pEntry->pDevc->pid)
      {
         if (dbg) syslog(LOG_DEBUG, "ulckDevc():Found it!!!");
         if (pEntry->pNext != NULL)
            pEntry->pNext->pPrev = pEntry->pPrev;
         if (pEntry->pPrev != NULL)
            pEntry->pPrev->pNext = pEntry->pNext;
         if (pEntry == pLockList)
            pLockList = pEntry->pNext;
         free(pEntry);
         break;
      }
   }
   if (dbg) syslog(LOG_DEBUG, "ulckDevc():Exit");
}

int isLocked(const char *pDevc)
{
   devcLock *pEntry;
   int rv = 0;

   if (dbg) syslog(LOG_DEBUG, "isLocked():Enter");
   for (pEntry = pLockList; pEntry != NULL; pEntry = pEntry->pNext)
   {
      if (dbg) syslog(LOG_DEBUG, "isLocked():Testing entries |%s| & |%s|",
                       pEntry->pDevc->devc, pDevc);
      if (strcmp(pEntry->pDevc->devc, pDevc) == 0)
      {
         if (dbg) syslog(LOG_DEBUG, "isLocked():Have Match!");
         rv = 1;
         break;
      }
   }
   if (dbg) syslog(LOG_DEBUG, "isLocked():Exit(%d)", rv);
   return(rv);
}
