/***************************************************************************
**    modem.c
**    modem device funtions
**
** Copyright (C) 1995, 1996  Joseph Croft <jcroft@unicomp.net>  
** 
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 1, or (at your option)
** any later version.
** 
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
** 
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software
** Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
**
 ***************************************************************************/
#include "../sysdefs.h"
#include <stdio.h>
#include <stdlib.h>
#include <signal.h>
#include <ctype.h>
#include <fcntl.h>
#include <unistd.h>
#include <termios.h>
#include "termsrvd.h"

#define DEBUG dbgout && dbg

static int dbg = 0;
extern FILE *dbgout;

extern int mfd;

#ifndef CRTSCTS
#define CRTSCTS (CTSFLOW | RTSFLOW)
#endif

int MdmDuplex = MDM_FULL;

static char MdmName[80];
static int MdmBaud = B38400;
static int MdmBits = 8;
static int MdmStop = 1;
static int MdmParity = MDM_NONE;
static int MdmSWCtl = 1;
static int MdmHWCtl = 0;


static struct termios MdmTermios;

void SetModem()
{
   int x;
   
   if (mfd)
   {
      x = tcgetattr(mfd, &MdmTermios);
/*      x = ioctl(mfd, TCGETA, &MdmTermios); */
      /*
      ** Start by sorting out the modem control bits (c_cflag)
      */
      x = MdmBaud | CREAD;
      
      if (MdmBits == 8)
         x |= CS8;
      else if (MdmBits == 7)
         x |= CS7;
      else if (MdmBits == 6)
         x |= CS6;
      else if (MdmBits == 5)
         x |= CS5;
         
      if (MdmParity == MDM_EVEN)
         x |= PARENB;
      else if (MdmParity == MDM_ODD)
         x |= PARENB | PARODD;

#ifndef AIX
      if (MdmHWCtl)
         x |= CRTSCTS;
#endif
      
      MdmTermios.c_cflag = x;

      MdmTermios.c_lflag = 0;
         
      MdmTermios.c_oflag = 0;	      /* Transparent output */

      if (MdmSWCtl)
         MdmTermios.c_iflag = IXON;
      MdmTermios.c_iflag |= IGNBRK;
      MdmTermios.c_cc[VMIN] = 0; 	   /* This many chars satisfies reads */
      MdmTermios.c_cc[VTIME] = 0;	   /* or in this many tenths of seconds */
      if (DEBUG) fprintf(dbgout, "SetModem():Calling ioctl\n\r");
      x = tcsetattr(mfd, TCSANOW, &MdmTermios);
/*      x = ioctl(mfd, TCSETAW, &MdmTermios); */
      if (DEBUG) fprintf(dbgout, "SetModem():ioctl returned %d\n\r", x);
   }
}

int openModem(char *name)
{
   char buf[128];
   int mflgs, rv = 1;
   
   strcpy(MdmName, name);
   
   if (mfd)
      close(mfd);
      
   if (DEBUG) fprintf(dbgout, "openModem():Attempting to open port |%s|\n\r", name);
   if ((mfd = open(MdmName, O_RDWR | O_NDELAY)) < 0)
   {
      if (DEBUG) fprintf(dbgout, "openModem():Error opening port |%s|\n\r", name);
      sprintf(buf, "Error opening modem port: %s\r", MdmName);
      perror(buf);
      mfd = 0;
      rv = 0;
   }
   else
   {
      if (DEBUG) fprintf(dbgout, "openModem():Success opening port |%s|, fileno = %d\n\r", name, mfd);
      if (((mflgs = fcntl(mfd, F_GETFL, 0)) == -1) ||
         (fcntl(mfd, F_SETFL, mflgs & ~O_NDELAY) == -1))
      {
         perror("Cannot Reset O_NDELAY on modem\r");
         rv = 0;
      }
      else
      {
         if (DEBUG) fprintf(dbgout, "openModem():Setting Port parameters\n\r");
         SetModem();
         rv = 1;
      }
   }
   return(rv);
}

void BuildModemParms(char *buf, int cbSize)
{
   char *cp;
   
   cp = buf;
   if (DEBUG) fprintf(dbgout, "BuildModemParms():MdmBits = %d\n\r", MdmBits);
   *(cp++) = MdmBits + '0';
   if (MdmParity == MDM_NONE)
      *(cp++) = 'N';
   else
      *(cp++) = (MdmParity == MDM_EVEN) ? 'E' : 'O';
      
   *(cp++) = MdmStop + '0';
   
   *(cp++) = (MdmDuplex == MDM_FULL) ? 'F' : 'H';
   
   *(cp++) = 'S';
   *(cp++) = (MdmSWCtl == 0) ? '0' : '1';
   
   *(cp++) = 'C';
   *(cp++) = (MdmHWCtl == 0) ? '0' : '1';
   
   *(cp) = '\0';
   if (DEBUG) fprintf(dbgout, "BuildModemParms():Buf = |%s|\n\r", buf);
}

void BuildModemBaud(char *buf, int cbSize)
{
   if (cbSize > 5)
   {
      switch (MdmBaud)
      {
         case B300:
            strcpy(buf, "300");
            break;
            
         case B600:
            strcpy(buf, "600");
            break;
            
         case B1200:
            strcpy(buf, "1200");
            break;
            
         case B2400:
            strcpy(buf, "2400");
            break;
            
         case B4800:
            strcpy(buf, "4800");
            break;
            
         case B9600:
            strcpy(buf, "9600");
            break;
            
         case B19200:
            strcpy(buf, "19200");
            break;
            
         case B38400:
            strcpy(buf, "38400");
            break;
            
         default:
            strcpy(buf, "");
            break;
      }
   }
}

void ShowModemSettings(char *pDevice)
{
   char buf[10];
   
   fprintf(dbgout, "Device: %s, ", pDevice);
   BuildModemBaud(buf, sizeof(buf));
   fprintf(dbgout, "Baud: %s, ", buf);
   fprintf(dbgout, "Bits: %c, ", MdmBits + '0');
   fprintf(dbgout, "Parity: %s, ", (MdmParity == MDM_NONE) ? "None" : 
                                                   (MdmParity == MDM_EVEN) ?
                                                    "Even" : "Odd");
   fprintf(dbgout, "Duplex: %s\n", (MdmDuplex == MDM_FULL) ? "Full" : "Half");
}

int parseBaudRate(uchar *cbuf, int cnt)
{
   unsigned char buf[128], *cp;
   int x, rv;
   
   if (DEBUG) fprintf(dbgout, "ParseModemBaud():enter\n\r");
   for (cp = buf, x = 0; x < cnt; x++)
      *(cp++) = *(cbuf++);
   *cp = '\0';
   
   x = atoi((char *)buf);
   
   switch (x)
   {
      case 300:
         x = B300;
         break;
   
      case 600:
         x = B600;
         break;
   
      case 1200:
         x = B1200;
         break;
   
      case 2400:
         x = B2400;
         break;
   
      case 4800:
         x = B4800;
         break;
   
      case 9600:
         x = B9600;
         break;
   
      case 19200:
         x = B19200;
         break;
   
      case 38400:
         x = B38400;
         break;
   
      default:
         x = -1;
   }
   if (x > 0)
   {
      MdmBaud = x;
      SetModem();
      rv = 1;
   }
   else
      rv = 0;
   if (DEBUG) fprintf(dbgout, "ParseModemBaud():exit\n\r");
   return(rv);
}

int parsePortSettings(uchar *cbuf, int cnt)
{
   int rv = 1;
   int bits = 0, parity = 0, duplex = 0, swctl = 0, hwctl = 0;
   int stop = 0, swset = 0, hwset = 0;
   
   while (cnt && *cbuf && rv != 0)
   {
      *cbuf = toupper(*cbuf);
      if (DEBUG) fprintf(dbgout, "ParseModemParms():Have char of 0x%x\n\r", *cbuf);
      switch(*cbuf)
      {
         case '8':
         case '7':
         case '6':
         case '5':
            if (DEBUG) fprintf(dbgout, "ParseModemParms():Setting Modem Byte size\n\r");
            bits = *cbuf - '0';
            if (DEBUG) fprintf(dbgout, "ParseModemParms():Modem Byte size = %d\n\r", bits);
            break; 
            
         case 'N':
            parity = MDM_NONE;
            break; 
            
         case 'E':
            parity = MDM_EVEN;
            break; 
            
         case 'O':
            parity = MDM_ODD;
            break; 
            
         case 'H':
            duplex = MDM_HALF;
            break; 
            
         case 'F':
            duplex = MDM_FULL;
            break;
            
         case 'S':
            if (*(++cbuf) == '0')
               swctl = 0;
            else
               swctl = 1;
            swset = 1;
            cnt--;
            break;
            
         case 'C':
            if (*(++cbuf) == '0')
               hwctl = 0;
            else
               hwctl = 1;
            hwset = 1;
            cnt--;
            break;

         case '1':
         case '2':
            stop = *cbuf - '0';
            break;
            
         default:
            rv = 0;
            break;
      }
      cbuf++;
      cnt--;
   }
   if (rv)
   {
      if (stop)
         MdmStop = stop;
      if (duplex)
         MdmDuplex = duplex;
      if (parity)
         MdmParity = parity;
      if (bits)
         MdmBits = bits;
      if (swset)
         MdmSWCtl = swctl;
      if (hwset)
         MdmHWCtl = hwctl;      
      SetModem();
   }
   return(rv);
}
/*
void _OutCh(char ch)
{
   write(mfd, &ch, 1);
   usleep(50000);
}   
 
void _OutStr(char *str)
{
   while (*str)
      _OutCh(*(str++));
}
*/
int SendBreak()
{
   tcsendbreak(mfd, 0);
/*   ioctl(mfd, TCSBRK,0); */
   sleep(1);
   tcsendbreak(mfd, 0);
/*   ioctl(mfd, TCSBRK,0); */
   sleep(1);
   return(tcsendbreak(mfd, 0));
/*   return(ioctl(mfd, TCSBRK,0)); */
}
