/***************************************************************************
**    termsrvd.c
**    termsrvd mainline code
**
** Copyright (C) 1995, 1996  Joseph Croft <jcroft@unicomp.net>  
** 
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 1, or (at your option)
** any later version.
** 
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
** 
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software
** Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
**
 ***************************************************************************/
#include "../sysdefs.h"
#include <arpa/telnet.h>
#include <sys/time.h>
#include <unistd.h>
#include <signal.h>
#include <errno.h>
#include "termsrvd.h"

#define DEBUG ((dbgout != NULL) && (dbg > 0))

extern int optind, opterr;
extern char *optarg;

extern int MdmDuplex;

char sBaudRate[256] = "38400";
char sPortDevice[256] = "";
char sPortSettings[256] = "8NF1";
dev_entry DeviceList[MAX_DEVICE + 1];

FILE *tnlin, *tnlout;
int nSynch;
int mfd = 0;
int iDevice;

int debug, verbose;
char debugFile[] = "/var/adm/termsrvd.log";
FILE *dbgout = NULL;

static int dbg = 0;
static FILE *mfp = NULL;

static fd_set ReadFds, ExcpFds;

void handleSpecialChars(int iVal, void *pArg1, void *pArg2)
{
   if (DEBUG) fprintf(dbgout, "Recieved special char %d\n\r", iVal);
   if (iVal == BREAK)
   {
      if (DEBUG) fprintf(dbgout, "Processing Break request\n\r");
      SendBreak();
   }
}

void handleTelNetErrors(int iVal, void *pArg1, void *pArg2)
{
   fprintf(stderr, "Recieved error %d, rv = %d\n\r", iVal, (int)pArg1);
   perror("");
}

void handleSubOptDataRequest(int iVal, void *pArg1, void *pArg2)
{
   char buf[256];
   
   if (DEBUG) fprintf(dbgout, "Have SubOption data resuest for %d\n\r", iVal);
   if (DEBUG) fflush(dbgout);
   if (iVal == TELOPT_NDEVICE)
   {
      if (DEBUG) fprintf(dbgout, "Resuest for NDEVICE\n\r");
      if (DEBUG) fflush(dbgout);
      
      iDevice++;
      if (iDevice > MAX_DEVICE)
         iDevice = 0;
      else if (DeviceList[iDevice].name == NULL)
      {
         if (DeviceList[MAX_DEVICE].dev == NULL)
            iDevice = 0;
         else
            iDevice = MAX_DEVICE;
      }
         
      if (DEBUG) fprintf(dbgout, "Next device in use: |%s|\n\r", DeviceList[iDevice].dev);
      if (DEBUG) fflush(dbgout);
      if (DeviceList[iDevice].name != NULL)
      {
         buildDeviceName(buf, &DeviceList[iDevice]); 
         if (DEBUG) fprintf(dbgout, "Sending back %s\n\r", buf);
         if (DEBUG) fflush(dbgout);
         tnlSetSubOption(iVal, buf);
      }
   }
   if (DEBUG) fflush(dbgout);
}

void handleNewSubOptData(int iVal, void *pArg1, void *pArg2)
{
   char fmtBuf[80], buff[80], *pDev;
   int x;
   
   if (DEBUG) fprintf(dbgout, "Have New SubOption data for %d\n\r", iVal);
   strcpy(fmtBuf, "");
   switch (iVal)
   {
      case TELOPT_BAUDRATE:
         strcpy(fmtBuf, "Baudrate set to: %s\n\r");
         if (parseBaudRate(pArg1, strlen(pArg1)))
            strcpy(sBaudRate, pArg1);
         strcpy(buff, sBaudRate);
         break;
         
      case TELOPT_PORTSET:
         strcpy(fmtBuf, "Port Settings are: %s\n\r");
         if (parsePortSettings(pArg1, strlen(pArg1)))
            strcpy(sPortSettings, pArg1);
         strcpy(buff, sPortSettings);
         break;
         
      case TELOPT_DEVICE:
         strcpy(fmtBuf, "Device is: %s\n\r");
         
         if (DEBUG) fprintf(dbgout, "Parsing for device %s\n\r", (char *)pArg1);
         if (DEBUG) fflush(dbgout);
         x = parseDeviceName(pArg1);
         if (x > 0 && x < MAX_DEVICE)
            pDev = DeviceList[x].dev;
         else if (x == MAX_DEVICE)
            pDev = pArg1;
         if (DEBUG) fprintf(dbgout, "Validating/Opening device %s\n\r", pDev);
         if (DEBUG) fflush(dbgout);
         if (x >= 0 && openModem(pDev))
         {
            iDevice = x;
            if (x < MAX_DEVICE)
            {
               if (DeviceList[x].port && strlen(DeviceList[x].port) > 0)
               {
                  if (DEBUG) fprintf(dbgout, "Setting Port Settings to %s\n\r", DeviceList[x].port);
                  if (DEBUG) fflush(dbgout);
                  strcpy(sPortSettings, DeviceList[x].port);
                  tnlSetSubOption(TELOPT_PORTSET, sPortSettings);
               }
               if (DeviceList[x].baud && strlen(DeviceList[x].baud) > 0)
               {
                  if (DEBUG) fprintf(dbgout, "Setting Baud Rate to %s\n\r", DeviceList[x].baud);
                  if (DEBUG) fflush(dbgout);
                  strcpy(sBaudRate, DeviceList[x].baud);
                  tnlSetSubOption(TELOPT_BAUDRATE, sBaudRate);
               }
            }

            if (strlen(sBaudRate) > 0)
            {
               x = parseBaudRate((uchar*)sBaudRate, strlen(sBaudRate));
               if (DEBUG) fprintf(dbgout, "Baud rate of %s is %s\n\r", sBaudRate,
                                 (x) ? "ok" : "not ok");
               if (DEBUG) fflush(dbgout);
            }
                                 
            if (strlen(sPortSettings) > 0)
            {
               x = parsePortSettings((uchar*)sPortSettings, strlen(sPortSettings));
               if (DEBUG) fprintf(dbgout, "Port settings of %s is %s\n\r", sPortSettings,
                                 (x) ? "ok" : "not ok");
               if (DEBUG) fflush(dbgout);
            }
         }
         buildDeviceName(buff, &DeviceList[iDevice]);
         break;
   }
   if (strlen(fmtBuf) > 0)
   {
      if (DEBUG) fprintf(dbgout, fmtBuf, (char *)pArg1);
      if (DEBUG) fflush(dbgout);
      if (DEBUG) fprintf(dbgout, "and was set to:\n\r");
      if (DEBUG) fflush(dbgout);
      if (DEBUG) fprintf(dbgout, fmtBuf, buff);
      if (DEBUG) fflush(dbgout);
      tnlSendSubOption(iVal, buff);
   }
   if (DEBUG) fflush(dbgout);
}

void doTermServ()
{
   int mError = 0, sflg = 0, cnt, x, rv;
   char commbuf[256];
   
   if (strlen(DeviceList[iDevice].dev) > 0)
      mError = 1;

   if (DEBUG) fprintf(dbgout, "Termnet:Initializing telnet library!\n\r");
   if ((rv = tnlInit(NULL, NULL, &tnlin, &tnlout, 15)) != -1)
   {
      if (DEBUG) fprintf(dbgout, "Termnet:Initialization complete!\n\r");
      /*
      ** Enable the proccessing of a few options & set the 
      ** Baud rate and port settings of the device
      */
      tnlSetCallBack(TNL_ISSUBOPTDATA_CB, handleNewSubOptData);
      tnlSetCallBack(TNL_SENDSUBOPTDATA_CB, handleSubOptDataRequest);
      tnlSetCallBack(TNL_SPECIALCHAR_CB, handleSpecialChars);
      tnlSetCallBack(TNL_ERROR_CB, handleTelNetErrors);
      tnlEnableOption(TELOPT_BAUDRATE);
      tnlEnableOption(TELOPT_PORTSET);
      tnlEnableOption(TELOPT_DEVICE);
      tnlEnableOption(TELOPT_NDEVICE);
      tnlEnableOption(TELOPT_ECHO);

      tnlSetSubOption(TELOPT_BAUDRATE, sBaudRate);
      tnlSetSubOption(TELOPT_PORTSET, sPortSettings);

      tnlOfferOption(TELOPT_ECHO, 1);
      tnlOfferOption(TELOPT_SGA, 1);
      tnlRequestOption(TELOPT_SGA, 1);

      for (;;)
      {
         if (mError && !mfd)
         {
            if (strlen(DeviceList[iDevice].dev) > 0)
            {
               if (openModem(DeviceList[iDevice].dev))
               {
                  buildDeviceName(commbuf, &DeviceList[iDevice]);
                  tnlSetSubOption(TELOPT_DEVICE, commbuf);
                  if (strlen(sBaudRate) > 0)
                  {
                     x = parseBaudRate((uchar*)sBaudRate, strlen(sBaudRate));
                     if (DEBUG) fprintf(dbgout, "Baud rate of %s is %s\n\r", sBaudRate,
                                       (x) ? "ok" : "not ok");
                  }
                                       
                  if (strlen(sPortSettings) > 0)
                  {
                     x = parsePortSettings((uchar*)sPortSettings, strlen(sPortSettings));
                     if (DEBUG) fprintf(dbgout, "Port settings of %s is %s\n\r", sPortSettings,
                                       (x) ? "ok" : "not ok");
                  }
                  
                  if ((mfp = fdopen(mfd, "w")) == NULL)
                     perror("Opening buffered serial device");   
               }
               else
               {
                  perror("Opening serial device");   
                  mfd = 0;
                  mfp = NULL;
               }
            }
         }
            
         FD_ZERO(&ReadFds);
         FD_ZERO(&ExcpFds);
         FD_SET(fileno(tnlin), &ReadFds);
         if (mfd)
            FD_SET(mfd, &ReadFds);
            
         fflush(dbgout);
         if ((sflg = tnlSelect(&ReadFds, NULL, &ExcpFds, NULL)) != 0)
         {
            if (DEBUG) fprintf(dbgout, "Returned from tnlSelect!!!\n\r");
            if (mfd && FD_ISSET(mfd, &ReadFds))
            {
               if (DEBUG) fprintf(dbgout, "Have input from modem\n\r");
               if ((cnt = read(mfd, commbuf, sizeof(commbuf))) > 0)
                  write(fileno(tnlout), commbuf, cnt);
               else if (cnt < 0 && errno != EAGAIN && errno != EINTR)
               {
                  mError = 1;
                  mfd = 0;
                  mfp = NULL;
               }
            }
            
            if (FD_ISSET(fileno(tnlin), &ReadFds))
            {
               if (DEBUG) fprintf(dbgout, "Have input from Network\n\r");
               if ((cnt = read(fileno(tnlin), commbuf, sizeof(commbuf))) > 0)
               {
                  if (DEBUG) 
                  {
                     fprintf(dbgout, "Read %d chars from Network\n\r", cnt);
                     for (x = 0; x < cnt; x++)
                        fprintf(dbgout, "%d, 0x%x\n", commbuf[x], commbuf[x]);
                  }
                  write(mfd, commbuf, cnt);
                  if (MdmDuplex == MDM_HALF)
                     write(fileno(tnlout), commbuf, cnt);
                  if (DEBUG) fprintf(dbgout, "Processed chars from Network\n\r");
               }
               else
               {
                  if (DEBUG) fprintf(dbgout, "Socket closed\n\r");
                  exit(-1);
               }
            }
            
            if (FD_ISSET(fileno(tnlin), &ExcpFds))
            {
               if (DEBUG) fprintf(dbgout, "Have Exception\n\r");
               nSynch = 1;
            }
         }
         if (DEBUG) fflush(dbgout);
      }
   }
   else
      perror("Error connecting");
}     

int main(int argc, char *argv[])
{
   int opt, x;

   iDevice = 0;
   dbgout = fopen(debugFile, "a");
   
   if (loadDevices() < 0)
   {
      fprintf(stderr, "Invalid or No Configuration file found!\n\r");
      exit(1);
   }
   strcpy(sPortDevice, DeviceList[0].dev);

   while ((opt = getopt(argc, argv, "b:d:p:Dv")) != -1)
   {
      switch (opt)
      {
         case 'b':
            strcpy(sBaudRate, optarg);
            break;
            
         case 'd':
            if ((x = parseDeviceName(optarg)) > 0)
            {
               iDevice = x;
               strcpy(sPortDevice, DeviceList[x].dev);
               if (x < MAX_DEVICE)
               {
                  if (DeviceList[x].port && strlen(DeviceList[x].port) > 0)
                  {
                     if (DEBUG) fprintf(dbgout, "Setting Port Settings to %s\n\r", DeviceList[x].port);
                     if (DEBUG) fflush(dbgout);
                     strcpy(sPortSettings, DeviceList[x].port);
                  }
                  if (DeviceList[x].baud && strlen(DeviceList[x].baud) > 0)
                  {
                     if (DEBUG) fprintf(dbgout, "Setting Baud Rate to %s\n\r", DeviceList[x].baud);
                     if (DEBUG) fflush(dbgout);
                     strcpy(sBaudRate, DeviceList[x].baud);
                  }
               }
            }
            else
            {
               fprintf(stderr, "Invalid device specified!\n\r");
               exit(1);
            }
            break;
            
         case 'p':
            strcpy(sPortSettings, optarg);
            break;
            
         case 'D':
            debug = 1;
            break;
            
         case 'v':
            verbose = 1;
            break;
            
         default:
            fprintf(dbgout, "Invalid command line option\n\r");
            fprintf(dbgout, "Usage: termsrvd [-b <baud rate>] [-d <port device>] [-p <port settings>] [-D] [-v]\n\r");
            exit(1);
      }
   }
   tnlSetSubOption(TELOPT_DEVICE, sPortDevice);
   doTermServ();
   exit(0);
}
