/***************************************************************************
**    tsdDevice.c
**    Termsrvd Device selection functions
**
** Copyright (C) 1995, 1996  Joseph Croft <jcroft@unicomp.net>  
** 
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 1, or (at your option)
** any later version.
** 
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
** 
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software
** Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
**
 ***************************************************************************/
#include "../sysdefs.h"
#include <string.h>
#include <ctype.h>
#include <stdlib.h>
#include <memory.h>
#include <stdio.h>
#include "termsrvd.h"

#define DEBUG dbgout && dbg

extern int iDevice;
extern dev_entry DeviceList[MAX_DEVICE + 1];
extern FILE *dbgout;

static int dbg = 0;

extern char sPortDevice[];
extern char sBaudRate[];
extern char sPortSettings[];

void buildDeviceName(char *buf, dev_entry *pDev)
{
   if (pDev->name == NULL || strlen(pDev->name) == 0)
      sprintf(buf, "%d:%s", pDev->id, pDev->dev);
   else
      sprintf(buf, "%d:%s:(%s)", pDev->id, pDev->name, pDev->dev);
}

int findDevice(char *dev)
{
   int x, rv;

   if (DEBUG) fprintf(dbgout, "findDevice():Validating device %s\n\r", dev);
   if (DEBUG) fflush(dbgout);
   for (rv = -1, x = 0; x < MAX_DEVICE && DeviceList[x].dev != NULL; x++)
   {
      if (DEBUG) fprintf(dbgout, "findDevice():Testing against dev %d: %s\n\r", x, DeviceList[x].dev);
      if (DEBUG) fflush(dbgout);
      if (strcmp(dev, DeviceList[x].dev) == 0)
      { 
         if (DEBUG) fprintf(dbgout, "findDevice():Match!!!!\n\r");
         if (DEBUG) fflush(dbgout);
         rv = x;
         break;
      }
   }
   if (DEBUG) fflush(dbgout);
   return(rv);
}

int parseDeviceName(char *dev)
{
   int rv, x;
   
   if (isdigit(*dev))
   {
      x = atoi(dev);
      if (x > 0 && x < MAX_DEVICE && DeviceList[x].dev != NULL)
         rv = x;
      else
         rv = -1;
   }
   else if (*dev == '/')
   {
      for (x = 0; x < MAX_DEVICE && DeviceList[x].name != NULL; x++)
      {
         if (strcmp(dev, DeviceList[x].dev) == 0)
         {
            if (DeviceList[MAX_DEVICE].dev)
               free(DeviceList[MAX_DEVICE].dev);
            if ((DeviceList[MAX_DEVICE].dev = (char *)malloc(strlen(dev) + 1)) != NULL)
               strcpy(DeviceList[MAX_DEVICE].dev, dev);
            x = -1;
            break;
         }
      }
      if (x < 0)
         rv = MAX_DEVICE;
      else
         rv = -1;
   }
   else
   {
      if (dev == NULL || strlen(dev) == 0)
         rv = -1;
      else
      {
         for (x = 0; x < MAX_DEVICE && DeviceList[x].name != NULL; x++)
            if (strcmp(dev, DeviceList[x].name) == 0)
               break;
         if (x < MAX_DEVICE)
            rv = x;
         else
            rv = -1;
      }
   }
   return(rv);
}
   
void setDevice()
{
   for (iDevice = 0; iDevice < MAX_DEVICE && DeviceList[iDevice].dev != NULL; iDevice++)
   {
      if (strcmp(sPortDevice, DeviceList[iDevice].dev) == 0)
         break;
   }
   if (iDevice >= MAX_DEVICE || DeviceList[iDevice].dev == NULL)
      iDevice = 0;
}

static int parseDeviceEntry(char *pDev, dev_entry *pEntry)
{
   char *cp;

   /*
   ** Split the entry into name and device,
   ** be sure to strip any trailing newlines
   */
   if ((cp = strtok(pDev, ":")) != NULL)
   {
      if (strlen(cp) > 0)
      {
         if ((pEntry->name = (char *)malloc(strlen(cp) + 1)) != NULL)
            strcpy(pEntry->name, cp);
      }
   }
   else
   {
      pEntry->name = NULL;
      return(0);
   }

   if ((cp = strtok(NULL, ":\n")) != NULL)
   {
      if (strlen(cp) > 0)
      {
         if ((pEntry->dev = (char *)malloc(strlen(cp) + 1)) != NULL)
            strcpy(pEntry->dev, cp);
      }
   }
   else
   {
      free(pEntry->name);
      pEntry->name = NULL;
      pEntry->dev = NULL;
      return(0);
   }
      
   if ((cp = strtok(NULL, ":\n")) != NULL)
   {
      if (strlen(cp) > 0)
      {
         if ((pEntry->baud = (char *)malloc(strlen(cp) + 1)) != NULL)
            strcpy(pEntry->baud, cp);
      }
   }
   else
   {
      if ((pEntry->baud = (char *)malloc(strlen(sBaudRate) + 1)) != NULL)
         strcpy(pEntry->baud, sBaudRate);
   }
      
   if ((cp = strtok(NULL, ":\n")) != NULL)
   {
      if (strlen(cp) > 0)
      {
         if ((pEntry->port = (char *)malloc(strlen(cp) + 1)) != NULL)
            strcpy(pEntry->port, cp);
      }
   }
   else
   {
      if ((pEntry->port = (char *)malloc(strlen(sPortSettings) + 1)) != NULL)
         strcpy(pEntry->port, sPortSettings);
   }
   return(1);
}

int loadDevices()
{
   FILE *fp;
   char buff[256];
   int x, rv = 0;
   
   for (x = 0; x <= MAX_DEVICE; x++)
   {
      DeviceList[x].id = x;
      DeviceList[x].name   = NULL;
      DeviceList[x].dev    = NULL;
      DeviceList[x].port   = NULL;
      DeviceList[x].baud   = NULL;
   }
   DeviceList[MAX_DEVICE].name = "<NULL>";

   if ((fp = fopen("/etc/termsrvd.conf", "r")) != NULL)
   {
      for (x = 0; x < MAX_DEVICE; x++)
      {
         if (fgets(buff, sizeof(buff), fp) != NULL)
         {
            DeviceList[x].id = x;
            if (!parseDeviceEntry(buff, &DeviceList[x]))
               break;
            if (DEBUG) fprintf(dbgout, "loadDevices():Added device: %s:%s:%s:%s\n\r",
                                        DeviceList[x].name,
                                        DeviceList[x].dev,
                                        DeviceList[x].port,
                                        DeviceList[x].baud);
         }
         else
            break;
      }
      if (x == 0)
         rv = -1;
   }
   else
      rv = -1;
   
   if (DeviceList[0].dev == NULL || DeviceList[0].name == NULL)
      rv = -1;
   return(rv);
}
